#!/bin/bash
echo "//this code is generated by gen_vector_vml.sh
#include \"../../../include/monolish_vml.hpp\"
#include \"../../internal/monolish_internal.hpp\"
#include \"./arithmetic/vecadd.hpp\"
#include \"./arithmetic/vecsub.hpp\"
#include \"./arithmetic/vecmul.hpp\"
#include \"./arithmetic/vecdiv.hpp\"
#include \"./math_2op/vecasin.hpp\"
#include \"./math_2op/vecasinh.hpp\"
#include \"./math_2op/vecatan.hpp\"
#include \"./math_2op/vecatanh.hpp\"
#include \"./math_2op/vecfloor.hpp\"
#include \"./math_2op/vecreciprocal.hpp\"
#include \"./math_2op/vecsign.hpp\"
#include \"./math_2op/vecsin.hpp\"
#include \"./math_2op/vecsinh.hpp\"
#include \"./math_2op/vecsqrt.hpp\"
#include \"./math_2op/vecceil.hpp\"
#include \"./math_2op/vectan.hpp\"
#include \"./math_2op/vectanh.hpp\"
#include \"./math_1_3op/vecmax.hpp\"
#include \"./math_1_3op/vecmin.hpp\"
#include \"./math_1_3op/vecpow.hpp\"

namespace monolish {
namespace vml {
"
## vector-vector arithmetic
funcs=(add sub mul div)
for func in ${funcs[@]}; do
  for prec in double float; do
    for arg1 in vector\<$prec\> view1D\<vector\<$prec\>,$prec\> view1D\<matrix::Dense\<$prec\>,$prec\>; do
      for arg2 in vector\<$prec\> view1D\<vector\<$prec\>,$prec\> view1D\<matrix::Dense\<$prec\>,$prec\>; do
        for arg3 in vector\<$prec\> view1D\<vector\<$prec\>,$prec\> view1D\<matrix::Dense\<$prec\>,$prec\>; do
          echo "void $func(const $arg1 &a, const $arg2 &b, $arg3 &y){vv${func}_core(a, b, y);}"
        done
      done
    done
  done
done

echo ""

## scalar-vector arithmetic
for func in ${funcs[@]}; do
  for prec in double float; do
    for arg1 in vector\<$prec\> view1D\<vector\<$prec\>,$prec\> view1D\<matrix::Dense\<$prec\>,$prec\>; do
      for arg2 in vector\<$prec\> view1D\<vector\<$prec\>,$prec\> view1D\<matrix::Dense\<$prec\>,$prec\>; do
        echo "void $func(const $arg1 &a, const $prec alpha, $arg2 &y){sv${func}_core(a, alpha, y);}"
      done
    done
  done
done

echo ""
#############################################

## vector-vector pow
for prec in double float; do
  for arg1 in vector\<$prec\> view1D\<vector\<$prec\>,$prec\> view1D\<matrix::Dense\<$prec\>,$prec\>; do
    for arg2 in vector\<$prec\> view1D\<vector\<$prec\>,$prec\> view1D\<matrix::Dense\<$prec\>,$prec\>; do
      for arg3 in vector\<$prec\> view1D\<vector\<$prec\>,$prec\> view1D\<matrix::Dense\<$prec\>,$prec\>; do
        echo "void pow(const $arg1 &a, const $arg2 &b, $arg3 &y){vvpow_core(a, b, y);}"
      done
    done
  done
done

echo ""

## scalar-vector pow
for prec in double float; do
  for arg1 in vector\<$prec\> view1D\<vector\<$prec\>,$prec\> view1D\<matrix::Dense\<$prec\>,$prec\>; do
    for arg2 in vector\<$prec\> view1D\<vector\<$prec\>,$prec\> view1D\<matrix::Dense\<$prec\>,$prec\>; do
      echo "void pow(const $arg1 &a, const $prec alpha, $arg2 &y){svpow_core(a, alpha, y);}"
    done
  done
done

echo ""
#############################################


## 2arg math
math=(sin sqrt sinh asin asinh tan tanh atan atanh ceil floor sign reciprocal)
for math in ${math[@]}; do
  for prec in double float; do
    for arg1 in vector\<$prec\> view1D\<vector\<$prec\>,$prec\> view1D\<matrix::Dense\<$prec\>,$prec\>; do
      for arg2 in vector\<$prec\> view1D\<vector\<$prec\>,$prec\> view1D\<matrix::Dense\<$prec\>,$prec\>; do
        echo "void $math(const $arg1 &a, $arg2 &y){v${math}_core(a, y);}"
      done
    done
  done
done

echo ""
#############################################

## vector-vector max min
funcs=(max min)
for func in ${funcs[@]}; do
  for prec in double float; do
    for arg1 in vector\<$prec\> view1D\<vector\<$prec\>,$prec\> view1D\<matrix::Dense\<$prec\>,$prec\>; do
      for arg2 in vector\<$prec\> view1D\<vector\<$prec\>,$prec\> view1D\<matrix::Dense\<$prec\>,$prec\>; do
        for arg3 in vector\<$prec\> view1D\<vector\<$prec\>,$prec\> view1D\<matrix::Dense\<$prec\>,$prec\>; do
          echo "void ${func[$i]}(const $arg1 &a, const $arg2 &b, $arg3 &y){vv${func}_core(a, b, y);}"
        done
      done
    done
  done
done

echo ""

## vector max min
funcs=(max min)
for func in ${funcs[@]}; do
  for prec in double float; do
    for arg1 in vector\<$prec\> view1D\<vector\<$prec\>,$prec\> view1D\<matrix::Dense\<$prec\>,$prec\>; do
      echo "$prec ${func[$i]}(const $arg1 &y){return v${func}_core<$arg1,$prec>(y);}"
    done
  done
done

echo ""
#############################################

echo "
} // namespace blas
} // namespace monolish "
